/** 
 ** This application reads the JPEG files in the indir and writes a 
 ** corresponding JPEG file in the outdir that has its pixel values reduced by 
 ** those in a.jpg. The absolute value of the difference in the color
 ** values for each pixel is stored in the outdir image. Files not ending
 ** in .jpg are ignored. 
 **  
 ** Usage: java JPGdiffAll a.jpg indir outdir
 **
 ** Author: L. Sweeney, For: Privacy and Anonymity in Data, Date: 10/2003.
 **/

import java.lang.reflect.Array;
import java.awt.*; 
import java.awt.event.*; 
import java.awt.image.*; 
import java.awt.geom.GeneralPath; 
import com.sun.image.codec.jpeg.*; 
import java.io.*; 
import javax.imageio.*; 


public class JPGdiffAll {

   /* This method returns an int containing a concatenated 24-bit value
    * of three 8-bit bytes, b followed by g followed by r.  Each of the
    * values b, g, and r are 8-bit values (0 to 255).
    */
   public static int getColor(int b, int g, int r) {
      return (int)Math.pow(2,16)*r + 256*g + b;
   }

   public static void main(String [] args) {
      try {

         //Read JPEG image that is to be substracted
         File f1 = new File(args[0]); 
         BufferedImage buf1 = ImageIO.read(f1);  
         System.out.println("JPEG file, " + args[0] + ", loaded.");
         Raster data1 = buf1.getData();
         DataBuffer datavg = data1.getDataBuffer();
         int width  = data1.getWidth();
         int height = data1.getHeight();

         File outdirname = new File(args[2]);
         File indirname = new File(args[1]);
         File [] files = indirname.listFiles();
         File f;
         String filename;
         BufferedImage bufi;  
         Raster data;
         DataBuffer datab;
         for(int i = 0; i < Array.getLength(files); i++) {

            //Read JPEG image from an input file 
            filename = indirname + "/" + files[i].getName();
            if( filename.endsWith(".jpg") ) {
               f = new File(filename); 
               bufi = ImageIO.read(f);  
               System.out.print("Loaded JPEG file: " + filename);
               data = bufi.getData();
               datab = data.getDataBuffer();

               if( (data.getWidth() == width) && (data.getHeight() == height)) {
                 int [] datav = new int[width*height*3];
                 //Compute pixel-level differences 
                 for(int j=0; j < datab.getSize(); j++) 
                    datav[j] = Math.abs(datab.getElem(j)-datavg.getElem(j));

                 // Convert 3 color bytes to a single 24-bit BGR pixel value.
                 int [] datanew = new int[width*height];
                 for(int pixpos=0, p=0; pixpos < width*height; pixpos++, p+=3)
                    datanew[pixpos] = getColor(datav[p], datav[p+1], datav[p+2]);
                 BufferedImage bufnew 
                    = new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR); 
                 bufnew.setRGB(0, 0, width, height, datanew, 0, width); 

                 //Make new JPEG file and save to output file 
                 filename = outdirname + "/" + files[i].getName();
                 File f2 = new File(filename);
                 ImageIO.write(bufnew, "jpg", f2);
                 System.out.println(". Saved dif to: " + filename);
               } else {
                     System.out.println("Skip " + filename + ". Bad dimensions.");
               }
             } else {
                System.out.println("Skip " + filename + ". Not end in .jpg.");
             }
          }
      } catch (Exception e) { 
            System.out.println("\nProblem encountered!");
            System.out.println("Usage: a.jpg indir outdir");
            System.out.println("Stores copies of .jpg files in indir to outdir ");
            System.out.println("with pixel values reduced by those in a.jpg."); 
      }
   }


}